/// @addtogroup taylor
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file tayltst.cpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <cmath>
#include <stdexcept>
#include <fstream>

#include "krak/krak.h"
#include "interval/doubleInterval.h"
#include "vectalg/vector.h"
#include "vectalg/matrix.h"
#include "vectalg/norm.h"

#include "map/function.h"
#include "map/map.h"
#include "map/c2map.h"

#include "dynsys/basicTaylor.h"
#include "dynsys/c2taylor.h"

#include "dynset/rect2set.h"
#include "dynset/rectset.h"
#include "dynset/intv2set.h"
#include "dynset/c2rect2.h"

const int DIMENSION=3;
typedef Vector<double,DIMENSION> FloatVector;
typedef Vector<interval,DIMENSION> IntervalVector;
typedef Matrix<double,DIMENSION,DIMENSION> FloatMatrix;
typedef Matrix<interval,DIMENSION,DIMENSION> IntervalMatrix;
typedef EuclLNorm<IntervalVector,IntervalMatrix> ELNorm;

typedef capd::map::Function<IntervalVector> Function;
typedef capd::map::Map<IntervalMatrix> Map;
typedef capd::map::C2Map<IntervalMatrix> C2Map;

typedef capd::dynsys::DynSys<IntervalMatrix> DynSys;
typedef capd::dynsys::Taylor<Map> Taylor;
typedef capd::dynsys::C2Taylor<C2Map> C2Taylor;
typedef capd::dynsys::BasicTaylor<capd::map::Map<FloatMatrix> > BasicTaylor;

typedef capd::dynset::C0set<IntervalMatrix> C0set;
typedef capd::dynset::RectSet<IntervalMatrix> RectSet;
typedef capd::dynset::Rect2Set<IntervalMatrix> Rect2Set;
typedef capd::dynset::Intv2Set<IntervalMatrix> Intv2Set;
typedef capd::dynset::C2rect2<IntervalMatrix> C2rect2;


double minx=-2.6;
double maxx=2.6;
double miny=-2.6;
double maxy=2.6;

Frame fr, txt;

void axes(Frame &fr)
{
   fr.line(0.0,miny,0.0,maxy,BLACK);
   fr.line(minx,0.0,maxx,0.0,BLACK);
}

// -------------------------------------------------------------------------

void initGraph(double step, int order)
{
   openGW(860,600,WHITE,BLACK);
   fr = Frame(5,115,345,455,WHITE,BLACK);
   txt = Frame(0,460,360,580,WHITE,BLACK);
   fr.setWorldCoord(minx,miny,maxx,maxy);
   rootFrame.Clear();
   axes(fr);
   rootFrame << "Test of set arithemtic using class \n\"Map\"and \"Taylor\".\n";
   rootFrame << "Evaluation of the set and its projection\n";
   rootFrame << "onto plane y\"=0.\n\n";
   rootFrame << "ODE: Kura-Siva\n";
   rootFrame << "time step: " << step << ",  order: " << order << "\n";
}

// -------------------------------------------------------------------------

void computeRigorousTrajectory(C0set &S, C2Taylor &T, int color, frstring description)
{
   int length=static_cast<int>( (8./T.getStep().rightBound()));

   txt.SetFgColor(color);
   txt << "interval test: " << description <<"\n";

   int i;
   for(i=0;i<length;i++)
   {
      IntervalVector w = IntervalVector(S);
      fr.boxFill(w[0].leftBound(),w[1].leftBound(),w[0].rightBound(),w[1].rightBound(),color);
      fr.box(w[0].leftBound(),w[1].leftBound(),w[0].rightBound(),w[1].rightBound());
      S.move(T);
   }
}

// -------------------------------------------------------------------------

void computeNonrigorousTrajectory(const FloatVector &center, double sizeOfSet, int grid)
{
   BasicTaylor T("var:x,y,z,c;fun:y,z,c^2-y-0.5*x*x;",6,0.1);
   T.setParameter("c",1.0);
   int length= static_cast<int>( (8./T.getStep()));

   txt.SetFgColor(BLUE);
   txt << "vector test: nonrigorous trajectory";
   int i,j;
   FloatVector left = center + FloatVector(0.,-sizeOfSet,0.);
   double step = 2*sizeOfSet/(double)grid;

   for(j=0;j<=grid;j++)
   {
      FloatVector P = left + FloatVector(0.,j*step,0.);
      for(i=0;i<length;i++)
      {
         fr.dot(P[0],P[1],BLUE);
         P = T(P);
      }
   }
}

// -------------------------------------------------------------------------

void C012Tests()
{
   rootFrame.Clear();
   rootFrame.precision(10);
   rootFrame << BgColor(BLACK) << FgColor(GREEN);
   rootFrame << "C^0 - Test\n";
   rootFrame << "The Kuramoto-Sivashinsky equation satisfies condition Q(t,X):=S(phi(t,S(phi(t,X))))= X\n";
   rootFrame << "wherever the left side is well defined. Take";

   interval step = 0.01;
   int order = 4;
   std::string f = "var:x,y,z;fun:y,z,1-y-0.5*x*x;";
   C2Taylor DS(f,order,step);
// C^0 - test
   IntervalVector X(interval(0.1),interval(1.5259617305037),interval(0.2));
   C2rect2 S1(X,ELNorm());
   int i,j,c,k,ss, numberOfSteps = 300;
   for(i=0;i<numberOfSteps;i++)
      S1.move(DS);

   IntervalVector FX = IntervalVector(S1);
   rootFrame << BgColor(WHITE) << FgColor(BLACK);
   rootFrame << "\n                    X=" << X << ", t=" << numberOfSteps*step;
   rootFrame << "\n             phi(t,X)=" << FX;
   FX[0] = -FX[0];
   FX[2] = -FX[2];
   rootFrame << "\n          S(phi(t,X))=" << FX;
   C2rect2 S2(FX,ELNorm());
   for(i=0;i<numberOfSteps;i++)
      S2.move(DS);
   FX = IntervalVector(S2);
   rootFrame << "\n   phi(t,S(phi(t,X)))=" << FX;
   FX[0] = -FX[0];
   FX[2] = -FX[2];
   rootFrame << "\nS(phi(t,S(phi(t,X))))=" << FX;
   if(X[0].subset(FX[0]) && X[1].subset(FX[1]) && X[2].subset(FX[2]))
   {
      rootFrame << BgColor(BLACK) << FgColor(YELLOW);
      rootFrame << "\nOK: X is a subset of Q(t,X)!";
   }
   else
   {
      rootFrame << BgColor(BLACK) << FgColor(RED);
      rootFrame << "\nFatal error: X is not a subset of Q(t,X)!";
   }
// C^1 - test
   rootFrame << BgColor(WHITE) << FgColor(BLACK);
   rootFrame << "\n\n-------------------------------------\n\n";
   rootFrame << BgColor(BLACK) << FgColor(GREEN);
   rootFrame << "C^1 - Test\n";
   rootFrame << BgColor(WHITE) << FgColor(BLACK);
   IntervalMatrix DP1 = IntervalMatrix(S1);
   IntervalMatrix DP2 = IntervalMatrix(S2);
   IntervalMatrix DQ; DQ.clear();

   bool ok = true;
   for(i=0;i<DIMENSION;i++)
      for(j=0;j<DIMENSION;j++)
      {
         for(k=0;k<DIMENSION;k++)
         {
            DQ[i][j]  += power(-1,k+1) * DP2[i][k] * DP1[k][j] ;
         }
         DQ[i][j] *= power(-1,i+1);
         if(i==j && !(DQ[i][j]).contains(1.)) ok = false;
         if(i!=j && !(DQ[i][j]).contains(0.)) ok = false;
      }
   rootFrame << DQ[0] << "\n";
   rootFrame << DQ[1] << "\n";
   rootFrame << DQ[2] << "\n";
   if(ok)
   {
      rootFrame << BgColor(BLACK) << FgColor(YELLOW);
      rootFrame << "OK: DQ(t,X) contains the Identity matrix!";
   }
   else
   {
      rootFrame << BgColor(BLACK) << FgColor(RED);
      rootFrame << "Fatal error: DQ(t,X) do not contain the Identity matrix!";
   }

// C^2 - test
   rootFrame << BgColor(WHITE) << FgColor(BLACK);
   rootFrame << "\n\n-------------------------------------\n\n";
   rootFrame << BgColor(BLACK) << FgColor(GREEN);
   rootFrame << "C^2 - Test";
   rootFrame << BgColor(WHITE) << FgColor(BLACK);

   IntervalMatrix D2Q[3];
   D2Q[0].clear(); D2Q[1].clear(); D2Q[2].clear();
   ok = true;
   for(i=0;i<DIMENSION;i++)
   {
      rootFrame << "\n";
      for(j=0;j<DIMENSION;j++)
      {
         for(c=0;c<DIMENSION;c++)
           {
            for(k=0;k<DIMENSION;k++)
            {
               D2Q[i][j][c] += power(-1,i+k) * DP2[i][k] * S1(k,j,c);
               for(ss=0;ss<DIMENSION;ss++)
               {
                  D2Q[i][j][c] += power(-1,i+k+ss+1) * S2(i,k,ss) * DP1[ss][c]*DP1[k][j];
               }
            }
            if(!D2Q[i][j][c].contains(0.)) ok = false;
         }
         rootFrame << D2Q[i][j] << "\n";
      }
   }

   if(ok)
   {
      rootFrame << BgColor(BLACK) << FgColor(YELLOW);
      rootFrame << "OK: The hessian matrices D^2Q(t,X) contain the zero matrix!";
   }
   else
   {
      rootFrame << BgColor(BLACK) << FgColor(RED);
      rootFrame << "Fatal error: The hessian matrices D^2Q(t,X) do not contain the zero matrix!";
   }
}

// -------------------------------------------------------------------------

int main(int, char *[])
{
   double step=0.06;
   int order = 8;
   initGraph(step,order);
   //init_fpunit();

   try
   {
      FloatVector z(0.,1.563,0.);
      IntervalVector v(0.,1.563,0.);
      double sizeOfSet = 0.005;

      interval iv(-sizeOfSet,sizeOfSet);
      IntervalVector r(0.,iv,0.);

      RectSet rect(v,r);
      Rect2Set rect2(v,r);
      Intv2Set intv2(v,r);

      C2Taylor T("var:x,y,z,c;fun:y,z,c^2-y-0.5*x*x;",order,step);
      T.setParameter("c",interval(1.0));

      computeRigorousTrajectory(rect,T,MAGENTA,"rectSet");
      computeRigorousTrajectory(intv2,T,RED,"intv2Set");
      computeRigorousTrajectory(rect2,T,GREEN,"rect2Set");
      computeNonrigorousTrajectory(z,sizeOfSet,4);
      waitBt();

      C012Tests();
      waitBt();
   }catch(std::exception& e)
   {

      std::ofstream plik;
      plik.open("report");
      plik << e.what();
      plik.close();

      rootFrame << "\n\nException caught! See 'report' file for details.";
      waitBt();
   }
   closeGW();
   return 0;
}


/// @}
