/// @addtogroup diffIncl
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file MultiMapIntf.h
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/* Author: Tomasz Kapela, 2007 */

#ifndef _MULTI_MAP_INTF_H_
#define _MULTI_MAP_INTF_H_

#include <string>
#include <map>
#include "vectalg/normintf.h"


namespace capd{
namespace diffIncl{

/////////////////////////////////////////////////////////////////////////////////
// MultiMap 
///
///  A multi map for differential inclusions
///
/// We assume that map is in the form
///   \f$ f(x,e) = f(x) + g(x,epsilon) \f$ 
/// where epsilon is some interval set,
/// e0 is contained in epsilon
/// and g(x,e0) = 0
///
/// This form is needed to estimate better the perturbations
/////////////////////////////////////////////////////////////////////////////////
template<typename MapT>
class MultiMap{

public:
  typedef MapT MapType;
  typedef typename MapType::FunctionType FunctionType;
//  typedef typename MapT::MultiMapType MultiMapType;
  typedef typename MapType::MatrixType MatrixType;
  typedef typename MatrixType::RowVectorType VectorType;
  typedef typename MatrixType::ScalarType ScalarType;
  typedef Norm<VectorType,MatrixType> NormType;
  typedef std::vector<ScalarType> ParamType;

  MultiMap( MapType& Field, 
            MapType& Perturb
  ) : m_f(Field), m_g(Perturb) {
  }

  
  /// value of MultiMap
  VectorType operator()(const VectorType & X) const {
    return m_f(X) + m_g(X);
  }

  /// derivative  of a MultiMap
  MatrixType operator[](const VectorType & X) const {
    return m_f[X] + m_g[X];
    
  }

  /// value and derivative of a MultiMap
  VectorType operator()(const VectorType & X, MatrixType &dF) const{
    dF = m_f[X] + m_g[X];
    return m_f(X) + m_g(X);
  }
  
  MapType & getVectorField(){
    return m_f;
  }
  
  MapType & getPerturbations(){
    return m_g;
  }
  
  VectorType perturbations(VectorType const & X){
    return m_g(X);
  }

protected:
  MapType & m_f;              ///< vector field 
  MapType & m_g;              ///< perturbations of the vector field
};



}} // namespace capd::diffIncl

#endif // define _TaylorClass

/// @}
