/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c2rect2intf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2003-2004 */

#ifndef _C2rect2Intf_H
#define _C2rect2Intf_H

#include "vectalg/normintf.h"
#include "map/c2coeffintf.h"
#include "dynset/c2set.h"
#include "dynsys/c2dynsys.h"

/* C^2-Lohner algorithm.
   Derivative of the flow moved via QR decomposition (third method)
   the set part - QR decomposition (3-rd method)
*/

namespace capd{
namespace dynset{

template<typename MatrixT>
class C2rect2: public C2set<MatrixT>{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;
   typedef capd::map::C2Coeff<ScalarType> C2CoeffType;

   C2rect2(const NormType& the_Norm, int dim);
   C2rect2(const VectorType& theX, const NormType&,double sFactor=20.);
   C2rect2(const VectorType& theX, const VectorType& the_r, const NormType&, double sFactor=20.);
   C2rect2(const VectorType& theX, const MatrixType&, const VectorType& the_r, const NormType&,double sFactor=20.);
   C2rect2(const C2rect2 &c);
   ~C2rect2();

   ScalarType size(void) const;
   C2set<MatrixType>* clone(void) const;
   C2set<MatrixType>* fresh(void) const;
   C2set<MatrixType>* cover(const VectorType &v) const;
   VectorType center(void) const;
   void move(capd::dynsys::C2DynSys<MatrixType>& c2dynsys);
   void move(capd::dynsys::C2DynSys<MatrixType>& c2dynsys, C2rect2& result);
   std::string show(void) const;
   const C2CoeffType& getC2Coeff() const {return m_c2Coeff;}
   NormType*getNorm(void);

   operator VectorType() const;
   operator MatrixType() const;
   const ScalarType& operator()(int i, int j, int c) const;
   VectorType operator()(int j, int c) const;
   MatrixType operator()(int i);
   C2rect2& operator=(const VectorType &v);
   C2rect2& operator=(const C2rect2 &S);

protected:
   int m_dim;

// a representation of the set: x + C*r0 + B*r

   VectorType m_x, m_r, m_r0;
   MatrixType m_B, m_C;
   MatrixType m_D, m_Bjac, m_R, m_Cjac, m_R0; // D + Bjac*R + Cjac*R0

// C^2 part represented as alpha + Bjac*Htilde
// C^2 part represented as alpha + Bjac*HR + Cjac*HR0

   C2CoeffType m_alpha, m_HR, m_HR0, m_c2Coeff;

   NormType *m_Norm; //the logarithmic Norm<VectorType,MatrixType> used in the computation of a rough
                     // enclosure for the variational part

   double sizeFactor;
};

// -----------------------------------------------------------------------------

template<typename MatrixType>
inline void C2rect2<MatrixType>::move(capd::dynsys::C2DynSys<MatrixType> &c2dynsys)
{
   move(c2dynsys,*this);
}

template<typename MatrixType>
inline C2set<MatrixType>* C2rect2<MatrixType>::clone(void) const
{
   return new C2rect2<MatrixType>(*this);
}

template<typename MatrixType>
inline C2set<MatrixType>* C2rect2<MatrixType>::fresh(void) const
{
   return new C2rect2<MatrixType>(*m_Norm,m_dim);
}

template<typename MatrixType>
inline C2set<MatrixType>* C2rect2<MatrixType>::cover(const VectorType &v) const
{
   return new C2rect2<MatrixType>(midVector(v),v-midVector(v), *m_Norm);
}

template<typename MatrixType>
inline typename C2rect2<MatrixType>::VectorType C2rect2<MatrixType>::center(void) const
{
   return m_x;
}

template<typename MatrixT>
inline C2rect2<MatrixT>::operator typename C2rect2<MatrixT>::VectorType(void) const
{
   return m_x + m_C*m_r0 + m_B*m_r;
}

template<typename MatrixType>
inline typename C2rect2<MatrixType>::NormType* C2rect2<MatrixType>::getNorm(void)
{
   return m_Norm;
}

}} // the end of the namespace capd::dynset

#endif // _C2rect2Intf_H

/// @}
