/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file flowball.hpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _FlowBall_HPP
#define _FlowBall_HPP

#include <stdexcept>
#include "dynset/flowballintf.h"
#include "vectalg/normintf.h"

namespace capd{
namespace dynset{

template<typename MatrixType>
typename FlowballSet<MatrixType>::ScalarType FlowballSet<MatrixType>::size(void)
{
   return ::size(intervalBall(m_x,m_r));
}

template<typename MatrixType>
void FlowballSet<MatrixType>::move(capd::dynsys::DynSys<MatrixType> &dynsys)
{
   if(dynsys.type() != "flow")
   {
      throw std::runtime_error("Attempt to use logarithmic norms with a cascade");
   }

   /*   radius is multipled by the Lipschitz constant of the generator of DynSys
      on the interval enclosure of the ball. Note that the proper computation of Lipschitz
      for flows is hidden in the implementation of Lipschitz for flows (class odenum)
   */

   VectorType s(m_x.dimension()), vi(m_x.dimension());
   m_r *= dynsys.Lipschitz(vi = intervalBall(m_x,m_r),*m_n);

   VectorType y=dynsys.Phi(m_x);
   split(y,s);
   s += dynsys.Remainder(m_x);
   m_x = y;
   m_r += (*m_n)(s);
}

template<typename MatrixType>
void FlowballSet<MatrixType>::move(capd::dynsys::DynSys<MatrixType> &dynsys, FlowballSet &result) const
{
   VectorType s(m_x.dimension()), vi(m_x.dimension());
   if(dynsys.type() != "flow")
   {
      throw std::runtime_error("Attempt to use logarithmic norms with a cascade");
   }

   /*   radius is multipled by the Lipschitz constant of the generator of DynSys
      on the interval enclosure of the ball. Note that the proper computation of Lipschitz
      for flows is hidden in the implementation of Lipschitz for flows (class odenum)
   */

   result.m_r = m_r*dynsys.Lipschitz(vi = intervalBall(m_x,m_r),*m_n);

   VectorType y = dynsys.Phi(m_x);
   split(y,s);
   s += dynsys.Remainder(m_x);
   result.m_x = y;
   result.m_r += (*m_n)(s);
   delete result.m_n;
   result.m_n = m_n->clone();
}

template<typename MatrixType>
std::string FlowballSet<MatrixType>::show(void) const
{
   std::ostringstream descriptor;
   descriptor << "FlowballSet(" << m_n->show() << "): x=";
   descriptor << m_x << " r=";
   descriptor << m_r << " ";
   return descriptor.str();
}

template<typename MatrixType>
typename FlowballSet<MatrixType>::VectorType FlowballSet<MatrixType>::affineTransformation(
      const MatrixType& A_M, const VectorType& A_C) const
{
   return A_M*(intervalBall(m_x,m_r)-A_C);
}

}} // namespace capd::dynset

#endif

/// @}
