/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file flowballintf.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _FlowBallIntf_H
#define _FlowBallIntf_H

#include <stdexcept>
#include "dynset/c0set.h"
#include "vectalg/normintf.h"

namespace capd{
namespace dynset{

template<typename MatrixT>
class FlowballSet : public C0set<MatrixT>{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;

   FlowballSet(): m_n(0){}
   FlowballSet(const NormType& the_n, int dim);
   FlowballSet(const VectorType& the_x, const ScalarType& the_r, const NormType& the_n);
   FlowballSet(const FlowballSet&);
   ~FlowballSet(){delete m_n;}
   virtual C0set<MatrixType>* clone(void);
   virtual C0set<MatrixType>* fresh(void);
   virtual C0set<MatrixType>* cover(const VectorType& v);
   virtual ScalarType size(void);
   virtual VectorType center(void);
   virtual void move(capd::dynsys::DynSys<MatrixType>& dynsys);
   virtual void move(capd::dynsys::DynSys<MatrixType>& dynsys, FlowballSet &result) const;
   virtual std::string show(void) const;
   virtual operator VectorType() const;
   virtual VectorType affineTransformation(const MatrixType&, const VectorType&) const;

protected:
   VectorType m_x;
   ScalarType m_r;
   NormType *m_n;
};

// inline definitions

template<typename MatrixT>
inline FlowballSet<MatrixT>::operator typename FlowballSet<MatrixT>::VectorType(void) const
{
   return intervalBall(m_x,m_r);
}

template<typename MatrixType>
inline typename FlowballSet<MatrixType>::VectorType FlowballSet<MatrixType>::center(void)
{
   return m_x;
}

template<typename MatrixType>
inline FlowballSet<MatrixType>::FlowballSet(
      const typename FlowballSet<MatrixType>::VectorType& the_x,
      const typename FlowballSet<MatrixType>::ScalarType& the_r,
      const typename FlowballSet<MatrixType>::NormType& the_n
   ) : m_x(the_x),m_r(the_r),m_n(the_n.clone())
{}

template<typename MatrixType>
inline C0set<MatrixType>* FlowballSet<MatrixType>::clone(void)
{
   return new FlowballSet<MatrixType>(*this);
}

template<typename MatrixType>
inline FlowballSet<MatrixType>::FlowballSet(const NormType& n, int dim)
   : m_x(dim), m_n(n.clone())
{}

template<typename MatrixType>
inline FlowballSet<MatrixType>::FlowballSet(const FlowballSet& S)
   : m_x(S.m_x), m_r(S.m_r), m_n(S.m_n->clone())
{}

template<typename MatrixType>
inline C0set<MatrixType>* FlowballSet<MatrixType>::fresh(void)
{
   return new FlowballSet<MatrixType>(*m_n,m_x.dimension());
}

template<typename MatrixType>
inline C0set<MatrixType>* FlowballSet<MatrixType>::cover(const VectorType &v)
{
   return new FlowballSet<MatrixType>(midVector(v),(*m_n)(rightVector(v)-midVector(v)),*m_n);
}

}} // namespace capd::dynset

#endif

/// @}
