/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file intv2setintf.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _Intv2SetIntf_H
#define _Intv2SetIntf_H

#include "dynset/c0set.h"

// C0set<IVectorType,IMatrixType>: x + C*r0 + r
// Lohner - internal representation;
//   C*r0 - Lipschitz part
//   r  -  'errors' computed via interval evaluation

namespace capd{
namespace dynset{

template<typename MatrixT>
class Intv2Set : public C0set<MatrixT> {
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;

   explicit Intv2Set(int dim);
   explicit Intv2Set(const VectorType& the_x);
   Intv2Set(const VectorType& the_x, const VectorType& the_r0);
   virtual C0set<MatrixType>* clone(void);
   virtual C0set<MatrixType>* fresh(void);
   virtual C0set<MatrixType>* cover(const VectorType &v);
   ScalarType size(void);
   virtual VectorType center(void);
   void move(capd::dynsys::DynSys<MatrixType>& dynsys);
   void move(capd::dynsys::DynSys<MatrixType>& dynsys, Intv2Set &result) const;
   operator VectorType() const;
   std::string show(void) const;
   virtual VectorType affineTransformation(const MatrixType&, const VectorType&) const;

protected:
   VectorType m_x, m_r, m_r0;
   MatrixType m_C;
};

// inline definitions

template<typename MatrixType>
inline typename Intv2Set<MatrixType>::VectorType Intv2Set<MatrixType>::center(void)
{
   return m_x;
}

template<typename MatrixT>
inline Intv2Set<MatrixT>::operator typename Intv2Set<MatrixT>::VectorType(void) const
{
   return m_x + m_C*m_r0 + m_r;
}

template<typename MatrixType>
inline Intv2Set<MatrixType>::Intv2Set(const VectorType& the_x)
   :  m_x(the_x),
      m_r(the_x.dimension()),
      m_r0(the_x.dimension()),
      m_C(MatrixType::Identity(the_x.dimension()))
{
   m_r.clear(); m_r0.clear();
}

template<typename MatrixType>
inline Intv2Set<MatrixType>::Intv2Set(int dim)
   :  m_x(dim),
      m_r(dim),
      m_r0(dim),
      m_C(MatrixType::Identity(dim))
{
   m_x.clear();
   m_r.clear();
   m_r0.clear();
}

template<typename MatrixType>
inline Intv2Set<MatrixType>::Intv2Set(const VectorType& the_x, const VectorType& the_r0)
   :  m_x(the_x),
      m_r(the_x.dimension()),
      m_r0(the_r0),
      m_C(MatrixType::Identity(the_x.dimension()))
{
   m_r.clear();
}

template<typename MatrixType>
inline C0set<MatrixType>* Intv2Set<MatrixType>::clone(void)
{
   return new Intv2Set<MatrixType>(*this);
}

template<typename MatrixType>
inline C0set<MatrixType>* Intv2Set<MatrixType>::fresh(void)
{
   return new Intv2Set<MatrixType>(m_x.dimension());
}

template<typename MatrixType>
inline C0set<MatrixType>* Intv2Set<MatrixType>::cover(const VectorType& v)
{
   return new Intv2Set<MatrixType>(midVector(v),v-midVector(v));
}

}} // namespace capd::dynset

#endif

/// @}
