/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file ppedsetintf.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _PpedSetIntf_H
#define _PpedSetIntf_H

#include "dynset/c0set.h"

namespace capd{
namespace dynset{

// C0set<IVectorType,IMatrixType> is represented as : x + B*r
// move via inverse matrix - Lohner second method

template<typename MatrixT>
class PpedSet : public C0set<MatrixT>{

public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;

   inline VectorType get_x () {return m_x;}
   inline VectorType get_r () {return m_r;}
   inline MatrixType get_B () {return m_B;}

   explicit PpedSet(int);
   explicit PpedSet(const VectorType& the_x);
   PpedSet(const VectorType& the_x, const VectorType& the_r);
   PpedSet(const VectorType& the_x, const VectorType& the_r, const MatrixType& the_B);
   ScalarType size(void);
   virtual C0set<MatrixType>* clone(void);
   virtual C0set<MatrixType>* fresh(void);
   virtual C0set<MatrixType>* cover(const VectorType& v);
   virtual VectorType center(void);
   void move(capd::dynsys::DynSys<MatrixType>& dynsys);
   void move(capd::dynsys::DynSys<MatrixType>& dynsys, PpedSet& result) const;
   virtual std::string show(void) const;
   operator VectorType() const;
   virtual VectorType affineTransformation(const MatrixType&, const VectorType&) const;

protected:
   VectorType m_x, m_r;
   MatrixType m_B;
};

// inline definitions

template<typename MatrixType>
inline PpedSet<MatrixType>::PpedSet(int dim)
   :  m_x(dim),
      m_r(dim),
      m_B(MatrixType::Identity(dim))
{
   m_x.clear();
   m_r.clear();
}

template<typename MatrixType>
inline typename PpedSet<MatrixType>::VectorType PpedSet<MatrixType>::center(void)
{
   return m_x;
}

template<typename MatrixT>
inline PpedSet<MatrixT>::operator typename PpedSet<MatrixT>::VectorType(void) const
{
   return m_x + m_B*m_r;
}

template<typename MatrixType>
inline PpedSet<MatrixType>::PpedSet(const VectorType& the_x)
   :  m_x(the_x),
      m_r(the_x.dimension()),
      m_B(MatrixType::Identity(the_x.dimension()))
{
   m_r.clear();
}

template<typename MatrixType>
inline PpedSet<MatrixType>::PpedSet(const VectorType& the_x,const VectorType& the_r)
   :  m_x(the_x),
      m_r(the_r),
      m_B(MatrixType::Identity(the_x.dimension()))
{}

template<typename MatrixType>
inline PpedSet<MatrixType>::PpedSet(
      const VectorType& the_x,
      const VectorType& the_r,
      const MatrixType& the_B
   ): m_x(the_x),
      m_r(the_r),
      m_B(the_B)
{}

template<typename MatrixType>
inline C0set<MatrixType>* PpedSet<MatrixType>::clone(void)
{
   return new PpedSet<MatrixType>(*this);
}

template<typename MatrixType>
inline C0set<MatrixType>* PpedSet<MatrixType>::fresh(void)
{
   return new PpedSet<MatrixType>(m_x.dimension());
}

template<typename MatrixType>
inline C0set<MatrixType>* PpedSet<MatrixType>::cover(const VectorType& v)
{
   return new PpedSet<MatrixType>(midVector(v),v-midVector(v));
}

}} // namespace capd::dynset

#endif

/// @}
