/// @addtogroup homologicalAlgebra
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file CrHomology.hpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.


#include <iostream>
#include <fstream>
#include <sstream>
#include <exception>
#include <stdexcept>
#include <new>
using namespace std;

#include "homologicalAlgebra/embeddingDim.h"
#include "auxil/ofstreamcout.h"
#include "auxil/Stopwatch.h"
#include "auxil/memSize.h"
#include "auxil/CRef.h"

extern ofstreamcout fcout;

#include "vectalg/matrixSlice.h"
#include "matrixAlgorithms/intMatrixAlgorithms.hpp"

using namespace capd;
using namespace matrixAlgorithms;

#include "homologicalAlgebra/homologicalAlgebra.hpp"
#include "homologicalAlgebra/readCubFile.hpp"
#include "homologicalAlgebra/readCubCellSet.hpp"
#include "homologicalAlgebra/homAlgFunctors.hpp"
#include "homologicalAlgebra/cubSetFunctors.hpp"
#include "homologicalAlgebra/ReducibleFreeChainComplex.hpp" // temporarily here, should be compiled into the library
#include "repSet/ECell3dCode.h"

void CrHom(int embDim, const int dims[], const char* bytes, int* betti,int streaming){
  if(streaming) fcout.turnOn();
  else fcout.turnOff();
  if(embeddingDim != embDim){
    std::ostringstream s;
    std::cout  << "This executable is for embedding dimension " << embeddingDim << " and the provided set has dimension " << embDim  << std::endl;
    s << "This executable is for embedding dimension " << embeddingDim << " and the provided set has dimension " << embDim  << std::endl;
    throw std::runtime_error(s.str());
  }
  if(dims[0] % 32) throw std::runtime_error("The 0th dimension must be a multiciplity of 32!");
  CRef<BCubSet> cubSetCR(new BCubSet(dims,bytes));
  CRef<HomologySignature> homSignCR=CubSetFunctors<BCubSet,BCubCelSet>::HomSignViaRepSetReductions(cubSetCR);
  for(int i=0;i<embeddingDim;++i) betti[i]=homSignCR().bettiNumber(i);
}

// Note: only one of the two following methods may be uncommented

// This method kept only for testing purposes for the above version of CrHom
/* // -------------- keep it here ------------
void CrHom(const char* fileName){
  int d[3];
  const int* dims=d;
  const char* bitmap=0;
  int betti[3];
  // The CR reference is stored in cubSetCR to avoid premature destroying of the bitmap
  CRef<BCubSet> cubSetCR=readCubFile(fileName,dims,bitmap);
  CrHom(embeddingDim,dims,bitmap,betti,false);
  for(int i=0;i<embeddingDim;++i) std::cout << "Betti[" << i << "]=" << betti[i]  << std::endl;
}
*/ // -------------- keep it here ------------

//typedef ECell3dCode ElementaryCellType;  // This in theory is quicker but the class is only for embDim=3 so far
typedef ElementaryCell ElementaryCellType;

typedef int ScalarType;
typedef Matrix<ScalarType,0,0> MatrixType;
typedef FreeModule<ElementaryCellType,MatrixType> FreeModuleType;
typedef FreeChainComplex<FreeModule<ElementaryCellType,MatrixType> > FreeChainComplexType;
typedef CubSetFunctors<BCubSet,BCubCelSet,FreeChainComplexType> CubSetFunctorsType;

void CrHom(const char* fileName){

//  CRef<BCubSet> cubSetCR=readCubFile<BCubSet>(fileName); // This version changed cubcellset to cubset and then again to cubcellset, which unnecessarily increased the size of the set
  CRef<BCubCelSet> cubCellSetCR=readCubCellSet<BCubSet,BCubCelSet>(fileName);
  // -- MM  std::cout << "Read cubCellSetCR()=\n" << cubCellSetCR() << std::endl;
/*
cubCellSetCR().rescale(2);
BCubCelSet cubCelSetCopy(cubCellSetCR()); // copy needed, because for some reason writing destroys the set
unsigned int repsetType='R'*256+'B'; // for representable sets
cubCelSetCopy.writeBmp("rescaled.bmd",repsetType);
std::cout << "Got " << cubCellSetCR().cardinality() << " cells" << std::endl;  // -- MM
*/
  Stopwatch swTot;
  CRef<HomologySignature> homSignCR=CubSetFunctorsType::HomSignViaRepSetReductions(cubCellSetCR);
//  CRef<HomologySignature> homSignCR=CubSetFunctorsType::HomSignViaRepSetReductions(cubSetCR);
//  CRef<HomologySignature> homSignCR=CubSetFunctorsType::HomSignViaRepSetReductionsNew(cubSetCR);
  fcout << "Computation completed in " << swTot  << std::endl;
  fcout << "Computed homology is: \n\n" << homSignCR()  << std::endl;
}
/// @}

