/// @addtogroup map
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c2map.hpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2001-2005 */

#ifndef _C2MAP_HPP
#define _C2MAP_HPP

#include <string>
#include <sstream>
#include <stdexcept>
#include <vector>
#include "map/textop.h"

namespace capd{
namespace map{

/*___________________________________________*/


template<typename MatrixType>
void C2Map<MatrixType>::computeSecondDerivatives(const VectorType& iv, C2CoeffType& result) const
{
   int i=0;
   typename C2CoeffType::iterator b=result.begin(), e=result.end();
   while(b!=e)
   {
      *b = c2der[i](iv);
      ++i;
      ++b;
   }
}

/*___________________________________________*/

template<typename MatrixType>
typename C2Map<MatrixType>::C2CoeffType*
   C2Map<MatrixType>::computeSecondDerivatives(VectorType iv[], int p) const
{
   int i,r;
   C2CoeffType* result = new (dim) C2CoeffType[p+1];
   for(r=0;r<=p;++r)
   {
      i=0;
      typename C2CoeffType::iterator b=result[r].begin(), e=result[r].end();
      while(b!=e)
      {
         *b = c2der[i](iv[r],r);
         ++i;
         ++b;
      }
   }
   return result;
}

/*___________________________________________*/

template<typename MatrixType>
C2Map<MatrixType>::C2Map(void)
{
   c2der = new FunctionType[dim];
}

/*___________________________________________*/

template<typename MatrixType>
C2Map<MatrixType>::C2Map(const char* s, int order) : Map<MatrixType>(s,order)
{
   allocateC2Der();
}

/*___________________________________________*/

template<typename MatrixType>
C2Map<MatrixType>::C2Map(const std::string& f, int order) : Map<MatrixType>(f,order)
{
   allocateC2Der();
}

/*___________________________________________*/

template<typename MatrixType>
C2Map<MatrixType>::C2Map(const C2Map<MatrixType>& m) : Map<MatrixType>(m)
{
   allocateC2Der();
}

/*_____________________________________________*/

template<typename MatrixType>
C2Map<MatrixType>& C2Map<MatrixType>::operator=(const char* s)
{
   Map<MatrixType>::operator=(s);
   delete[]c2der;
   allocateC2Der();
   return *this;
}

/*_______________________________________________*/

template<typename MatrixType>
C2Map<MatrixType>& C2Map<MatrixType>::operator=(const std::string &f)
{
   Map<MatrixType>::operator=(f);
   delete[]c2der;
   allocateC2Der();
   return *this;
}

/*________________________________________________*/

template<typename MatrixType>
C2Map<MatrixType>& C2Map<MatrixType>::operator=(const C2Map<MatrixType> &m)
{
   Map<MatrixType>::operator=(m);
   delete []c2der;
   allocateC2Der();
   return *this;
}

/*________________________________________________*/


template<typename MatrixType>
void C2Map<MatrixType>::setOrder(int the_new)
{
   Map<MatrixType>::setOrder(the_new);
   for(int i=0;i<dim*dim*(1+dim)/2;++i)
      c2der[i].setOrder(the_new);
}

/*________________________________________________*/

template<typename MatrixType>
void C2Map<MatrixType>::setParameter(const std::string &name, const ScalarType &value)
{
   Map<MatrixType>::setParameter(name,value);
   for(int i=0;i<dim*dim*(1+dim)/2;++i)
   {
      c2der[i].setParameter(name,value);
   }
}

/*________________________________________________*/

template<typename MatrixType>
void C2Map<MatrixType>::allocateC2Der()
{
   c2der = new FunctionType[dim*dim*(dim+1)/2];
   int i,j,k;
   for(i=0;i<dim;++i)
      for(j=0;j<dim;++j)
         for(k=0;k<=j;k++)
         {
            c2der[i*dim*(dim+1)/2 + j*(j+1)/2 + k] = der[i*dim+j][k];
            c2der[i*dim*(dim+1)/2 + j*(j+1)/2 + k].setOrder(order);
         }

}

/*________________________________________________*/

}} // namespace capd::map

#endif // #define _C2MAP_HPP

/// @}
